from distutils.core import setup

setup(
    name='periodics',
    version='0.1.2',
    packages=['periodics'],
    url='',
    license='GPL',
    author='yarnaid',
    author_email='yarnaid@gmail.com',
    description='Some statistical functions for searchin periodics in time rows'
)
